unit OleWord;

{
OleWord.pas shows how to call MS-Word 6 / 7 with a document that is
stored in a BLOB-field of a table.

It uses the Application.OnActivate-event to tell the user
'Please close the document in MS-Word first' if he switches to
the application without closing the document in MS-Word.

It adds an 'Go-Back-button' in MS-Word and clears it after the
document is closed - but this is implemented for the german version
of MS-Word only - as many things in this unit.

Don't be confused about the short 'CTS' - it's the name of the
application I wrote this for.

Don't be confused about the word 'Seriendruck' - I just havn't found
the english word for this mass-letter thing in any dictionary.

You will see some '$IFNDEF WIN32' - but I don't think the 16-Bit-part
will still work - too many changes are made with Delphi 2 without
using this unit in Delphi 1 again.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Forms, DBTables,
{$IFNDEF WIN32}
  ToCtrl, OleCtnrs;
{$ELSE NDEF WIN32}
  OleCtnrs;
{$ENDIF NDEF WIN32}

type

  TMSWordLanguage = (wlUnknown, wlEnglish, wlGerman, wlFrench);
  TMsWordActivateAction =  { what to do if user comes back from Winword: }
  (waaNothing,
   waaCloseDocument,       { close the document in Winword }
   waaBackToWord,          { go back to Winword if document is still open }
   waaMinimizeWordIfEmpty, { minimize Winword if no document is open }
   waaMinimizeWord);       { minimize Winword anyway }
  TMsWordActivateActionSet = set of TMsWordActivateAction;

  TMsWordAfterEditEvent = procedure (Sender: TObject; Ok: Boolean) of object;

  TCreateDocumentFromFileRetInfo = record
    FileName: String;
    FileDate: TDateTime;
  end;
  PCreateDocumentFromFileRetInfo = ^TCreateDocumentFromFileRetInfo;

  TMsWord = Class(TObject)
  private
    FOwner: TForm;
    FAppOnActivate: TNotifyEvent;
    FAppOnDeactivate: TNotifyEvent;
    FMsWordAfterEditEvent: TMsWordAfterEditEvent;
    FOLEMsWord: Variant;
    FOLEContainer: TOLEContainer;
    FMsWordActivateActionSet: TMsWordActivateActionSet;
    FMSWordLanguage: TMSWordLanguage;
    FSerienbrief: Boolean;
    function PrepareMSWord: Boolean;
    function GetOLEMsWord: Variant;
    function GetMSWordLanguage: TMSWordLanguage;
    function GetOLEContainer: TOLEContainer;
    function GetModified: Boolean;
    procedure SetModified(Value: Boolean);
    procedure AppOnActivate(Sender: TObject);
  public
    constructor Create(aOwner: TForm);
    destructor Destroy; override;
    function InsertGoBackButton: Boolean;
    function DeleteGoBackButton: Boolean;
    function CloseOurDocument: Boolean;
    function MinimizeWord: Boolean;
    function ActivateWord: Boolean;
    function SeriendruckOpenDatasource(Datasource: String): Boolean;
    function CallWordToEditSeriendruckDocument(
      Datasource: String;
      ActivateAction: TMsWordActivateActionSet;
      MsWordAfterEditEvent: TMsWordAfterEditEvent): Boolean;
    function CallWordToEditDocument(
      ActivateAction: TMsWordActivateActionSet;
      MsWordAfterEditEvent: TMsWordAfterEditEvent): Boolean;
    function LoadFromBlobField(BlobField: TBlobField): Boolean;
    function SaveToBlobField(BlobField: TBlobField): Boolean;
    function CreateDocumentFromFile(const FileDialogTitle: String;
      var InitialDir: String;
      CreateDocumentFromFileRetInfo: PCreateDocumentFromFileRetInfo): Boolean;
    property OLEMsWord: Variant read GetOLEMsWord;
    property MSWordLanguage: TMSWordLanguage read GetMSWordLanguage;
    property OLEContainer: TOLEContainer read GetOLEContainer;
    property Modified: Boolean read GetModified write SetModified;
  end;


implementation

uses OleAuto, Dialogs;

const
{ GoBackButtonAction = 'AnwAktivieren'; {}
  GoBackButtonAction = 'BackToCTS'; {}
  ApplicationName = 'CTS';

{ The first letters in the title of this application;
  MS-Word uses the title of a window to find and activate it: }
  ApplicationCapture = ApplicationName;


constructor TMsWord.Create(aOwner: TForm);
begin
  inherited Create;
  FOLEMsWord := Unassigned;
  FMSWordLanguage := wlUnknown;
  FOLEContainer := nil;
  FOwner := aOwner;
  FSerienbrief := false;
end;

destructor TMsWord.Destroy;
begin
{
  if Assigned(FOLEContainer) then
    FOLEContainer.Free;
}
  inherited Destroy;
end;


function TMsWord.GetOLEMsWord: Variant;
begin
{ if (FOLEMsWord = Unassigned) then... does not work }
  if (FMSWordLanguage = wlUnknown) then
    PrepareMSWord;
  GetOLEMsWord := FOLEMsWord;
end;


function TMsWord.GetMSWordLanguage: TMSWordLanguage;
var
  s: String;
begin
{ if (FOLEMsWord = Unassigned) then... does not work  }
  if (FMSWordLanguage = wlUnknown) then
    PrepareMSWord
  else
  begin { check if Word is still running: }
    try
      case FMSWordLanguage of
        wlEnglish, wlFrench: s := FOLEMsWord.AppInfo(16);
        wlGerman: s := FOLEMsWord.AnwInfo(16);
      end;
    except
      try
        FOLEMsWord := CreateOleObject('Word.Basic');   { start again }
      except
        ShowMessage('Could not start Microsoft Word.');
      end;
    end;
  end;
  GetMSWordLanguage := FMSWordLanguage;
end;


function TMsWord.GetOLEContainer: TOLEContainer;
begin
  if not Assigned(FOLEContainer) then
  begin
    FOLEContainer := TOLEContainer.Create(FOwner);
    FOLEContainer.Name := 'MsWordOLEContainer';
    FOLEContainer.Top := FOwner.Width + 1;
    FOLEContainer.Parent := FOwner;
    FOLEContainer.OldStreamFormat := true;
    FOLEContainer.AllowInPlace := false;
    FOLEContainer.Caption := ApplicationName;
  end;
  GetOLEContainer := FOLEContainer;
end;


function TMsWord.PrepareMSWord: Boolean;
var
  sMSWordLanguage: string;
begin
  result := false;
  try
    FOLEMsWord := CreateOleObject('Word.Basic');
  except
    ShowMessage('Could not start Microsoft Word.');
    Exit;
  end;
  try
  { Return Application Info. }
  { for German Microsoft Word the procedure call is translated }
    sMSWordLanguage := FOLEMsWord.AnwInfo(16);
  except
    try
    { This call is the same for English and French Microsoft Word. }
      sMSWordLanguage := FOLEMsWord.AppInfo(16);
    except
     { if this procedure does not exist there is a different translation of
       Microsoft Word }
       ShowMessage('Microsoft Word version is not German, French or English.');
       Exit;
    end;
  end;
  if (sMSWordLanguage = 'English (US)') or (sMSWordLanguage = 'English (UK)') then
  begin
    FMSWordLanguage := wlEnglish;
    result := true;
  end;
  if sMSWordLanguage = 'Franais' then
  begin
    FMSWordLanguage := wlFrench;
    result := true;
  end;
  if (sMSWordLanguage = 'German (De)') or (sMSWordLanguage = 'Deutsch') then
  begin
    FMSWordLanguage := wlGerman;
    result := true;
  end;
end;

function TMsWord.InsertGoBackButton: Boolean;
var
  AnzMakros: Integer;
  MakroNummer: Integer;
  Found: Boolean;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    ShowMessage('English version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    try
{     OLEMsWord.ExtrasMakro('StartFromCTS', True); {}
      Found := false;
      AnzMakros := OLEMsWord.ZhlenMakros(0);
      for MakroNummer := 1 to AnzMakros do
        if (OLEMsWord.MakroName(MakroNummer) = GoBackButtonAction) then
        begin
          Found := true;
          break;
        end;
      if not Found then
      begin { Makro erstellen: }
        OLEMsWord.ExtrasMakro(Name := GoBackButtonAction, Bearbeiten := 1, Beschreibung := 'Go back to ' + ApplicationName);
        OLEMsWord.Einfgen('AnwAktivieren "' + ApplicationCapture + '"');
        OLEMsWord.DateiSchlieen(1);
      end;
      if (OLEMsWord.SymbolleistenSchaltflchenMakro('Standard', 1, 0) <> GoBackButtonAction) then
      begin
        OLEMsWord.SchaltflcheHinzufgen('Standard', 1, 1, GoBackButtonAction, 'Back to ' + ApplicationName, 0, ApplicationName); {}
        OLEMsWord.SchaltflcheHinzufgen('Standard', 2, 1, ''); {}
      end;
      result := true;
    except
      result := false;
    end;
  end;
end;


function TMsWord.DeleteGoBackButton: Boolean;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    ShowMessage('English version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    try
      if (OLEMsWord.SymbolleistenSchaltflchenMakro('Standard', 1, 0) = GoBackButtonAction) then
      begin
        OLEMsWord.SchaltflcheLschen('Standard', 1); {}
        if (OLEMsWord.SymbolleistenSchaltflchenMakro('Standard', 1, 0) = '') then
        begin
          OLEMsWord.SchaltflcheLschen('Standard', 1); {}
        end;
      end;
      result := true;
    except
      result := false;
    end;
  end;
end;


function TMsWord.CloseOurDocument: Boolean;
var
  AnzFenst: Integer;
  FensterNummer: Integer;
  FenName: String;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    ShowMessage('English version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    try
      AnzFenst := OLEMsWord.ZhlenFenster;
      for FensterNummer := 1 to AnzFenst do
      begin
        FenName := OLEMsWord.FensterName(FensterNummer);
        if (Pos(ApplicationName, FenName) > 0) then
        begin { gefunden: }
          OLEMsWord.FensterListe(FensterNummer); { aktivieren }
          if FSerienbrief then
          begin
            OLEMsWord.SeriendruckLsen;
            FSerienbrief := false;
          end;
          OLEMsWord.DateiSchlieen;
          break;
        end;
      end;
      result := true;
    except
      result := false;
    end;
  end;
end;


function TMsWord.SeriendruckOpenDatasource(Datasource: String): Boolean;
const
  UmwandlungBestaetigen = 0;
  Schreibgeschuetzt = 0;
  QuelleVerknuepfen = 1;
  ZuletztBearbErweitern = 0;
  KennwortDok = '';
  KennwortDokVorlage = '';
  Wiederherstellen = 0;
  KennwortDokSchreiben = '';
  KennwortDokVorlageSchreiben = '';
var
  AnzFenst: Integer;
  FensterNummer: Integer;
  FenName: String;
  sVerbindung: String;
  sSQLAnweisung: String;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    ShowMessage('English version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    sVerbindung := 'DSN=dBASE-Dateien;DBQ=' +
      ExtractFileDir(Datasource) + ';DriverId=533;UID=admin;';
    sSQLAnweisung := 'SELECT * FROM ' +
      ChangeFileExt(ExtractFileName(Datasource), '');
    try
      AnzFenst := OLEMsWord.ZhlenFenster;
      for FensterNummer := 1 to AnzFenst do
      begin
        FenName := OLEMsWord.FensterName(FensterNummer);
        if (Pos(ApplicationName, FenName) > 0) then
        begin { gefunden: }
          OLEMsWord.FensterListe(FensterNummer); { aktivieren }
          if (OLEMsWord.SeriendruckHauptdokumentArt = -1) then
            OLEMsWord.SeriendruckHauptdokumentArt(0);
          OLEMsWord.SeriendruckffnenDatenquelle(
            Datasource,
            UmwandlungBestaetigen,
            Schreibgeschuetzt,
            QuelleVerknuepfen,
            ZuletztBearbErweitern,
            KennwortDok,
            KennwortDokVorlage,
            Wiederherstellen,
            KennwortDokSchreiben,
            KennwortDokVorlageSchreiben,
            sVerbindung,
            sSQLAnweisung);
          break;
        end;
      end;
      result := true;
    except
      result := false;
    end;
  end;
end;


function TMsWord.MinimizeWord: Boolean;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    ShowMessage('English version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    try
      OLEMsWord.AnwMinimieren(1);
      result := true;
    except
      result := false;
    end;
  end;
end;


function TMsWord.ActivateWord: Boolean;
begin
  result := false;
  if (MSWordLanguage = wlEnglish) then
  begin
    try
      OLEMsWord.AppActivate;
      result := true;
    except
      result := false;
    end;
  end;
  if (MSWordLanguage = wlFrench) then
  begin
    ShowMessage('French version of Microsoft Word is not supported yet.');
    exit;
  end;
  if (MSWordLanguage = wlGerman) then
  begin
    try
      OLEMsWord.AnwAnzeigen;
      result := true;
    except
      result := false;
    end;
  end;
end;

function TMsWord.CallWordToEditSeriendruckDocument(
  Datasource: String;
  ActivateAction: TMsWordActivateActionSet;
  MsWordAfterEditEvent: TMsWordAfterEditEvent): Boolean;
begin
  result := false;
  if CallWordToEditDocument(ActivateAction, MsWordAfterEditEvent) then
    if SeriendruckOpenDatasource(Datasource) then
    begin
      FSerienbrief := true;
      result := true;
    end
    else
    begin
      Application.OnActivate := FAppOnActivate;
      DeleteGoBackButton;
      if Assigned(FMsWordAfterEditEvent) then
        FMsWordAfterEditEvent(self, false);
      FSerienbrief := false;
    end;
end;

function TMsWord.CallWordToEditDocument(ActivateAction: TMsWordActivateActionSet;
  MsWordAfterEditEvent: TMsWordAfterEditEvent): Boolean;
var
  OldCursor: HCursor;
begin
  result := false;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  FSerienbrief := false;
  FMsWordActivateActionSet := ActivateAction;
  OLEContainer.Refresh;
{$IFDEF WIN32}
  if (OLEContainer.State = osLoaded) then
  begin
    InsertGoBackButton;
    FAppOnActivate := Application.OnActivate;
    Application.OnActivate := AppOnActivate;
    FMsWordAfterEditEvent := MsWordAfterEditEvent;
    OLEContainer.Modified := false;
    try
      OLEContainer.DoVerb(ovShow); { show document }
      result := true;
    except
      Application.OnActivate := FAppOnActivate;
      SetCursor(OldCursor);
      raise;
    end;
  end;
{$ELSE DEF WIN32}
  if MsWord.OLEContainer.OLEObjAllocated then
  begin
    FAppOnActivate := Application.OnActivate;
    Application.OnActivate := AppOnActivate;
    FMsWordAfterEditEvent := MsWordAfterEditEvent;
    OLEContainer.Modified := false;
    OLEContainer.Active := true; { show document }
    result := true;
  end;
{$ENDIF DEF WIN32}
  SetCursor(OldCursor);
end;

(*
  (waaNothing,
   waaCloseDocument,       { close the document in Winword }
   waaBackToWord,          { go back to Winword if document is still open }
   waaMinimizeWordIfEmpty, { minimize Winword if no document is open }
   waaMinimizeWord);       { minimize Winword anyway }
  TMsWordActivateActionSet = set of TMsWordActivateAction;
*)

procedure TMsWord.AppOnActivate(Sender: TObject);
var
  OldCursor: HCursor;
begin
  if Assigned(FAppOnActivate) then
    FAppOnActivate(Sender);
  Application.OnActivate := FAppOnActivate;
{$IFDEF WIN32}
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  if (waaCloseDocument in FMsWordActivateActionSet) and
     (OLEContainer.State <> osLoaded) then
    CloseOurDocument;
  if (OLEContainer.State <> osLoaded) then
  begin
    try
      ShowMessage('Please close the document in MS-Word first.');
      Application.OnActivate := AppOnActivate; { try again later }
      ActivateWord; { set Focus to MS-Word }
    except
      Application.OnActivate := FAppOnActivate;
      DeleteGoBackButton;
      if Assigned(FMsWordAfterEditEvent) then
        FMsWordAfterEditEvent(self, false);
      FSerienbrief := false;
    end;
  end
  else
  begin
    if DeleteGoBackButton then
      if (waaMinimizeWord in FMsWordActivateActionSet) or
         (waaMinimizeWordIfEmpty in FMsWordActivateActionSet) then
        MinimizeWord;
    if Assigned(FMsWordAfterEditEvent) then
      FMsWordAfterEditEvent(self, true);
    FSerienbrief := false;
  end;
  SetCursor(OldCursor);
{$ELSE DEF WIN32}
  dlgWaitTilDocSaved := TdlgWaitTilDocSaved.Create(self);
  dlgWaitTilDocSaved.ShowModal;
  dlgWaitTilDocSaved.Free;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  if Assigned(FMsWordAfterEditEvent) then
    FMsWordAfterEditEvent(self, true);
  SetCursor(OldCursor);
{$ENDIF DEF WIN32}
end;


function TMsWord.LoadFromBlobField(BlobField: TBlobField): Boolean;
var
  BS: TBlobStream;
  OldCursor: HCursor;
begin
  result := false;
  if (BlobField.IsNull) then
  begin
    exit;
  end;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  BS := TBlobStream.Create(BlobField, bmRead);
  try
    OLEContainer.LoadFromStream(BS);
    result := true;
  finally
    BS.Free;
  end;
  SetCursor(OldCursor);
end;


function TMsWord.SaveToBlobField(BlobField: TBlobField): Boolean;
var
  BS: TBlobStream;
  OldCursor: HCursor;
begin
  result := false;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  BS := TBlobStream.Create(BlobField, bmWrite);
  try
    OLEContainer.SaveToStream(BS);
    result := true;
  finally
    BS.Free;
  end;
  SetCursor(OldCursor);
end;


function TMsWord.GetModified: Boolean;
begin
  if not Assigned(FOLEContainer) then
    result := false
  else
    if (FOLEContainer.State = osEmpty) then
      result := false
    else
      result := FOLEContainer.Modified;
end;


procedure TMsWord.SetModified(Value: Boolean);
begin
  if Assigned(OLEContainer) then
    if (OLEContainer.State <> osEmpty) then
      OLEContainer.Modified := Value;
end;


function TMsWord.CreateDocumentFromFile(
  const FileDialogTitle: String; var InitialDir: String;
  CreateDocumentFromFileRetInfo: PCreateDocumentFromFileRetInfo): Boolean;
var
{$IFDEF WIN32}
{$ELSE DEF WIN32}
  Info: Pointer;
{$ENDIF DEF WIN32}
  OldCursor: HCursor;
  OpenDialog_ChooseLetter: TOpenDialog;
begin
  result := false;
  if (CreateDocumentFromFileRetInfo <> nil) then
    with CreateDocumentFromFileRetInfo^ do
    begin
      FileName := '';
      FileDate := 0;
    end;
{$IFDEF WIN32}
{ same as: if not MsWord.OLEContainer.InsertObjectDialog then exit...
  - but create *.doc from file only: }
  OpenDialog_ChooseLetter := TOpenDialog.Create(FOwner);
  OpenDialog_ChooseLetter.Options := [ofHideReadOnly,ofPathMustExist,ofFileMustExist];
  OpenDialog_ChooseLetter.Filter := 'Word document (*.doc)|*.doc|All files|*.*';
  OpenDialog_ChooseLetter.FilterIndex := 1;
  OpenDialog_ChooseLetter.Filename := '';
  OpenDialog_ChooseLetter.InitialDir := InitialDir;
  OpenDialog_ChooseLetter.Title := FileDialogTitle;
  if (not OpenDialog_ChooseLetter.Execute) or
     (OpenDialog_ChooseLetter.Filename = '') then
  begin
    OpenDialog_ChooseLetter.Free;
    exit;
  end;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  try
    OLEContainer.CreateObjectFromFile(AnsiUpperCase(OpenDialog_ChooseLetter.FileName), False);
    InitialDir := ExtractFilePath(OpenDialog_ChooseLetter.FileName);
    if (CreateDocumentFromFileRetInfo <> nil) then
      with CreateDocumentFromFileRetInfo^ do
      begin
        FileName := OpenDialog_ChooseLetter.FileName;
        FileDate := FileDateToDateTime(FileAge(OpenDialog_ChooseLetter.FileName));
      end;
    result := true;
  except
    result := false;
  end;
  OpenDialog_ChooseLetter.Free;
  SetCursor(OldCursor);
{$ELSE DEF WIN32}
  if not InsertOLEObjectDlg(self, 0, Info) then
    exit;
  OldCursor := SetCursor(LoadCursor(0, IDC_WAIT));
  OLEContainer.PInitInfo := Info;
  ReleaseOLEInitInfo(Info);
  SetCursor(OldCursor);
{$ENDIF DEF WIN32}
end;




end.
