{-----------------------------------------------------------------------------}
{ A component to read version info resources.  It is intended for Delphi 3,   }
{ but should work with any file that contains a properly formatted resource.  }
{ Copyright 1996, Brad Stowers & Greg Nixon.  All Rights Reserved.            }
{ This component can be freely used and distributed in commercial and private }
{ environments, provied this notice is not modified in any way and there is   }
{ no charge for it other than nomial handling fees.  Contact me directly for  }
{ modifications to this agreement.                                            }
{-----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions  }
{ at bstowers@pobox.com or 72733,3374 on CompuServe.                          }
{ The lateset version will always be available on the web at:                 }
{   http://www.pobox.com/~bstowers/delphi/                                    }
{-----------------------------------------------------------------------------}
{ Date last modified:  August 18, 1997                                        }
{-----------------------------------------------------------------------------}


{-----------------------------------------------------------------------------}
{ TVersionInfoResource v2.00                                                  }
{-----------------------------------------------------------------------------}
{ Description:                                                                }
{   A component to read version info resources.  It is intended for Delphi 3, }
{   but should work with any file that contains a properly formatted resource.}
{ Notes:                                                                      }
{   * I have not tested this on anything but Delphi 3 generated EXEs with     }
{     proper version info resources, and on Triplex generated 16-bit version  }
{     info resources.                                                         }
{   * The CreationDate property is faked under Delphi 1.  If anyone has a     }
{     function, or even pseudo-code, for converting the dwFileDateLS and MS   }
{     variables into TDateTime format, please send it to me.  For now, I just }
{     get the date/time stamp of the file containing the ver info resource.   }
{-----------------------------------------------------------------------------}
{ Revision History:                                                           }
{ 1.00:  + Initial release.                                                   }
{ 1.10:  + 16-bit compatibility. (not released as 2.00 version came too fast) }
{ 2.00:  + In a nutshell, Greg Nixon (gnixon@whanganui.ac.nz) added the       }
{          CopyrightLabel, DescriptionLabel, ProductLabel, VersionLabel and   }
{          VersionGrid properties. I tweaked and fiddled with some of it, but }
{          he gets the credit for the work.                                   }
{        + Changed FileVersion and ProductVersion to be a new class type.     }
{          This class type will let you check individual values, as well as   }
{          get the entire version as a string.  The properties are:           }
{            Major, Minor, Release, Build, AsString                           }
{          AsString is built like this:                                       }
{            <major>.<minor>.<release>.<build>  ex.  "2.1.0.391"              }
{          Thanks to Stefan de Vogelaere (Stefan.de.Vogelaere@telenet.be) for }
{          this idea.                                                         }
{-----------------------------------------------------------------------------}

unit VersInfo;

interface

uses
  {$IFDEF WIN32}
  Windows,
  {$ELSE}
  WinTypes, WinProcs, Ver,
  {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Grids,
  StdCtrls;

{$IFDEF WIN32}
  {$R VERSINFO.R32}
{$ELSE}
  {$R VERSINFO.R16}
{$ENDIF}

const
  IDX_COMPANYNAME           = 0;
  IDX_FILEDESCRIPTION       = 1;
  IDX_FILEVERSION           = 2;
  IDX_INTERNALNAME          = 3;
  IDX_LEGALCOPYRIGHT        = 4;
  IDX_LEGALTRADEMARKS       = 5;
  IDX_ORIGINALFILENAME      = 6;
  IDX_PRODUCTNAME           = 7;
  IDX_PRODUCTVERSION        = 8;
  IDX_COMMENTS              = 9;
  IDX_BUILDFLAGS            = 10;

const
  IDX_VER_MAJOR   = 0;
  IDX_VER_MINOR   = 1;
  IDX_VER_RELEASE = 2;
  IDX_VER_BUILD   = 3;


type

  {$IFNDEF WIN32}
  PVSFixedFileInfo = PVS_FixedFileInfo;
  DWORD = longint;
  UINT = word;
  {$ENDIF}


  TFixedFileInfoFlag = (ffDebug, ffInfoInferred, ffPatched, ffPreRelease,
      ffPrivateBuild, ffSpecialBuild);
  TFixedFileInfoFlags = set of TFixedFileInfoFlag;

  TVersionOperatingSystemFlag = (vosUnknown, vosDOS, vosOS2_16, vosOS2_32,
      vosNT, vosWindows16, vosPresentationManager16, vosPresentationManager32, vosWindows32);
  { This is supposed to be one of the first line, and one of the second line. }
  TVersionOperatingSystemFlags = set of TVersionOperatingSystemFlag;

  TVersionFileType = (vftUnknown, vftApplication, vftDLL, vftDriver, vftFont,
      vftVXD, vftStaticLib);

  TVersionInfoResource = class; { forward declaration }

  TFixedFileVersionInfo = class
  private
    FParent: TVersionInfoResource;
    FData: PVSFixedFileInfo;

    function GetSignature: DWORD;
    function GetStructureVersion: DWORD;
    function GetFileVersionMS: DWORD;
    function GetFileVersionLS: DWORD;
    function GetProductVersionMS: DWORD;
    function GetProductVersionLS: DWORD;
    function GetValidFlags: TFixedFileInfoFlags;
    function GetFlags: TFixedFileInfoFlags;
    function GetFileOperatingSystem: TVersionOperatingSystemFlags;
    function GetFileType: TVersionFileType;
    function GetFileSubType: DWORD;
    function GetCreationDate: TDateTime;
  public
    constructor Create(AParent: TVersionInfoResource);

    property Parent: TVersionInfoResource
      read FParent write FParent;
    property Data: PVSFixedFileInfo
      read FData write FData;

    property Signature: DWORD
      read GetSignature;
    property StructureVersion: DWORD
      read GetStructureVersion;
    property FileVersionMS: DWORD
      read GetFileVersionMS;
    property FileVersionLS: DWORD
      read GetFileVersionLS;
    property ProductVersionMS: DWORD
      read GetProductVersionMS;
    property ProductVersionLS: DWORD
      read GetProductVersionLS;
    property ValidFlags: TFixedFileInfoFlags
      read GetValidFlags;
    property Flags: TFixedFileInfoFlags
      read GetFlags;
    property FileOperatingSystem: TVersionOperatingSystemFlags
      read GetFileOperatingSystem;
    property FileType: TVersionFileType
      read GetFileType;
    property FileSubType: DWORD
      read GetFileSubType;
    property CreationDate: TDateTime
      read GetCreationDate;
  end;

  TVersionNumberInformation = class
  private
    FMostSignificant: DWORD;
    FLeastSignificant: DWORD;

    function GetVersionNumber(Index: integer): word;
    function GetVersionNumberString: string;
  public
    constructor Create(MSVer, LSVer: DWORD);

    property Major: word
      index IDX_VER_MAJOR
      read GetVersionNumber;
    property Minor: word
      index IDX_VER_MINOR
      read GetVersionNumber;
    property Release: word
      index IDX_VER_RELEASE
      read GetVersionNumber;
    property Build: word
      index IDX_VER_BUILD
      read GetVersionNumber;

    property AsString: string
      read GetVersionNumberString;
  end;

  TVersionInfoResource = class(TComponent)
  private
    FVersionInfo: PChar;
    FVersionInfoSize: DWORD;
    FFilename: string;
    FTranslationIDs: TStringList;
    FTranslationIDIndex: integer;
    FFixedInfo: TFixedFileVersionInfo;
    FForceEXE: boolean;
    FFileVersion: TVersionNumberInformation;
    FProductVersion: TVersionNumberInformation;
    FFileVersionLabel: TLabel;
    FCopyrightLabel: TLabel;
    FVersionGrid: TStringGrid;
    FDescriptionLabel: TLabel;
    FProductLabel: TLabel;
  protected
    procedure SetFilename(const Val: string);
    procedure SetTranslationIDIndex(Val: integer);
    function GetTranslationIDs: TStrings;
    procedure SetForceEXE(Val: boolean);
    procedure SetVersionGrid(Value: TStringGrid);
    procedure SetFileVersionLabel(Value: TLabel);
    procedure SetCopyrightLabel(Value: TLabel);
    procedure SetProductLabel(Value: TLabel);
    procedure SetDescriptionLabel(Value: TLabel);

    function GetResourceFilename: string; virtual;
    procedure PopulateControls; virtual;
    procedure BuildGrid; virtual;
    procedure ReadVersionInfoData; virtual;

    function GetVersionInfoString(Index: integer): string;
    function GetResourceStr(Index: string): string;

    procedure Notification(AComponent: TComponent;
                     Operation: TOperation); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property TranslationIDIndex: integer
      read FTranslationIDIndex
      write SetTranslationIDIndex;
    property TranslationIDs: TStrings
      read GetTranslationIDs;
    property FixedInfo: TFixedFileVersionInfo
      read FFixedInfo;
    property UserResource[Index: string]: string
      read GetResourceStr;

    property FileVersion: TVersionNumberInformation
      read FFileVersion;
    property ProductVersion: TVersionNumberInformation
      read FProductVersion;
  published
    property Filename: string
      read FFilename
      write SetFilename;
    property ForceEXE: boolean
      read FForceEXE
      write SetForceEXE
      default FALSE;

    property CompanyName: string
      index IDX_COMPANYNAME
      read GetVersionInfoString;
    property FileDescription: string
      index IDX_FILEDESCRIPTION
      read GetVersionInfoString;
    property InternalName: string
      index IDX_INTERNALNAME
      read GetVersionInfoString;
    property LegalCopyright: string
      index IDX_LEGALCOPYRIGHT
      read GetVersionInfoString;
    property LegalTrademarks: string
      index IDX_LEGALTRADEMARKS
      read GetVersionInfoString;
    property OriginalFilename: string
      index IDX_ORIGINALFILENAME
      read GetVersionInfoString;
    property ProductName: string
      index IDX_PRODUCTNAME
      read GetVersionInfoString;
    property Comments: string
      index IDX_COMMENTS
      read GetVersionInfoString;

    property VersionLabel: TLabel
      read FFileVersionLabel
      write SetFileVersionLabel;
    property VersionGrid: TStringGrid
      read FVersionGrid
      write SetVersionGrid;
    property DescriptionLabel: TLabel
      read FDescriptionLabel
      write SetDescriptionLabel;
    property CopyrightLabel: TLabel
      read FCopyrightLabel
      write SetCopyrightLabel;
    property ProductLabel: TLabel
      read FProductLabel
      write SetProductLabel;
  end;


procedure Register;


implementation


const
  PREDEF_RESOURCES: array[IDX_COMPANYNAME..IDX_BUILDFLAGS] of string = (
     'CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTrademarks', 'OriginalFilename', 'ProductName',
     'ProductVersion', 'Comments', 'BuildFlags'
    );


procedure Register;
begin
  RegisterComponents('My Controls', [TVersionInfoResource]);
end;


{$IFDEF VER90}
function SystemTimeToDateTime(const SystemTime: TSystemTime): TDateTime;
begin
  with SystemTime do
    Result := EncodeDate(wYear, wMonth, wDay) +
      EncodeTime(wHour, wMinute, wSecond, wMilliSeconds);
end;
{$ENDIF}

{$IFNDEF WIN32}
function IsLibrary: boolean;
begin
  Result := PrefixSeg = 0;
end;
{$ENDIF}


constructor TFixedFileVersionInfo.Create(AParent: TVersionInfoResource);
begin
  inherited Create;
  FParent := AParent;
end;

function TFixedFileVersionInfo.GetSignature: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwSignature;
end;

function TFixedFileVersionInfo.GetStructureVersion: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwStrucVersion;
end;

function TFixedFileVersionInfo.GetFileVersionMS: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwFileVersionMS;
end;

function TFixedFileVersionInfo.GetFileVersionLS: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwFileVersionLS;
end;

function TFixedFileVersionInfo.GetProductVersionMS: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwProductVersionMS;
end;

function TFixedFileVersionInfo.GetProductVersionLS: DWORD;
begin
  if FData = NIL then
    Result := 0
  else
    Result := FData^.dwProductVersionLS;
end;

function TFixedFileVersionInfo.GetValidFlags: TFixedFileInfoFlags;
begin
  Result := [];
  if FData <> NIL then
  begin
    if (FData^.dwFileFlagsMask and VS_FF_DEBUG) <> 0 then
      Include(Result, ffDebug);
    if (FData^.dwFileFlagsMask and VS_FF_PRERELEASE) <> 0 then
      Include(Result, ffPreRelease);
    if (FData^.dwFileFlagsMask and VS_FF_PATCHED) <> 0 then
      Include(Result, ffPatched);
    if (FData^.dwFileFlagsMask and VS_FF_PRIVATEBUILD) <> 0 then
      Include(Result, ffPrivateBuild);
    if (FData^.dwFileFlagsMask and VS_FF_INFOINFERRED ) <> 0 then
      Include(Result, ffInfoInferred );
    if (FData^.dwFileFlagsMask and VS_FF_SPECIALBUILD ) <> 0 then
      Include(Result, ffSpecialBuild );
  end;
end;

function TFixedFileVersionInfo.GetFlags: TFixedFileInfoFlags;
begin
  Result := [];
  if FData <> NIL then
  begin
    if (FData^.dwFileFlags and VS_FF_DEBUG) <> 0 then
      Include(Result, ffDebug);
    if (FData^.dwFileFlags and VS_FF_PRERELEASE) <> 0 then
      Include(Result, ffPreRelease);
    if (FData^.dwFileFlags and VS_FF_PATCHED) <> 0 then
      Include(Result, ffPatched);
    if (FData^.dwFileFlags and VS_FF_PRIVATEBUILD) <> 0 then
      Include(Result, ffPrivateBuild);
    if (FData^.dwFileFlags and VS_FF_INFOINFERRED ) <> 0 then
      Include(Result, ffInfoInferred );
    if (FData^.dwFileFlags and VS_FF_SPECIALBUILD ) <> 0 then
      Include(Result, ffSpecialBuild );
  end;
end;

function TFixedFileVersionInfo.GetFileOperatingSystem: TVersionOperatingSystemFlags;
{$IFNDEF WIN32}
var
  FileOS: word;
{$ENDIF}
begin
  Result := [];
  if FData <> NIL then
  begin
    case HiWord(FData^.dwFileOS) of
      VOS_DOS shr 16:   Include(Result, vosDOS);
      VOS_OS216 shr 16: Include(Result, vosOS2_16);
      VOS_OS232 shr 16: Include(Result, vosOS2_32);
      VOS_NT shr 16:    Include(Result, vosNT);
    else
      Include(Result, vosUnknown);
    end;

{$IFDEF WIN32}
    case LoWord(FData^.dwFileOS) of
      LoWord(VOS__WINDOWS16): Include(Result, vosWindows16);
      LoWord(VOS__PM16):      Include(Result, vosPresentationManager16);
      LoWord(VOS__PM32):      Include(Result, vosPresentationManager32);
      LoWord(VOS__WINDOWS32): Include(Result, vosWindows32);
    else
      Include(Result, vosUnknown);
    end;
{$ELSE}
    FileOS := LoWord(FData^.dwFileOS);
    if      FileOS = LoWord(VOS__WINDOWS16) then Include(Result, vosWindows16)
    else if FileOS = LoWord(VOS__PM16) then      Include(Result, vosPresentationManager16)
    else if FileOS = LoWord(VOS__PM32) then      Include(Result, vosPresentationManager32)
    else if FileOS = LoWord(VOS__WINDOWS32) then Include(Result, vosWindows32)
    else                                         Include(Result, vosUnknown);
{$ENDIF}
  end;
end;

function TFixedFileVersionInfo.GetFileType: TVersionFileType;
begin
  Result := vftUnknown;
  if FData <> NIL then
  begin
    case FData^.dwFileType of
      VFT_APP:        Result := vftApplication;
      VFT_DLL:        Result := vftDLL;
      VFT_DRV:        Result := vftDriver;
      VFT_FONT:       Result := vftFont;
      VFT_VXD:        Result := vftVXD;
      VFT_STATIC_LIB: Result := vftStaticLib;
    end;
  end;
end;

function TFixedFileVersionInfo.GetFileSubType: DWORD;
begin
  if FData = NIL then
    Result := 0
  else begin
    Result := FData^.dwFileSubtype;
  end;
end;

function TFixedFileVersionInfo.GetCreationDate: TDateTime;
{$IFDEF WIN32}
var
  SysTime: TSystemTime;
  FileTime: TFileTime;
begin
  if FData = NIL then
    Result := 0
  else begin
    FileTime.dwLowDateTime := FData^.dwFileDateLS;
    FileTime.dwHighDateTime := FData^.dwFileDateMS;
    if FileTimeToSystemTime(FileTime, SysTime) then
    begin
      Result := SystemTimeToDateTime(SysTime);
    end else
      Result := 0;
  end;
{$ELSE}
var
  SR: TSearchRec;
begin
  { Fake it until I can figure out how to convert dwFileDateMS and LS }
  Result := 0;
  if assigned(FParent) then
  begin
    if FindFirst(FParent.GetResourceFilename, faAnyFile, SR) = 0 then
    begin
      Result := FileDateToDateTime(SR.Time);
      FindClose(SR);
    end;
  end;
(*
var
  BigNum: comp;
begin
  if FData = NIL then
    Result := 0
  else begin

    BigNum := (FData^.dwFileDateMS * MaxLongInt) + FData^.dwFileDateLS;
    BigNum := BigNum / 10000000;
  { LS and MS is the number of 100 nanosecond intervals since 1/1/1601 }
  { 10,000,000s of a second }
    Result := EncodeDate(1601, 1, 1);
    Result := BigNum.....
  end;
*)

{$ENDIF}
end;



constructor TVersionNumberInformation.Create(MSVer, LSVer: DWORD);
begin
  inherited Create;

  FMostSignificant := MSVer;
  FLeastSignificant := LSVer;
end;

function TVersionNumberInformation.GetVersionNumber(Index: integer): word;
begin
  case Index of
    IDX_VER_MAJOR:   Result := HiWord(FMostSignificant);
    IDX_VER_MINOR:   Result := LoWord(FMostSignificant);
    IDX_VER_RELEASE: Result := HiWord(FLeastSignificant);
    IDX_VER_BUILD:   Result := LoWord(FLeastSignificant);
  else
    Result := 0;
  end;
end;

function TVersionNumberInformation.GetVersionNumberString: string;
begin
  Result := Format('%d.%d.%d.%d', [Major, Minor, Release, Build]);
end;



constructor TVersionInfoResource.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FVersionInfo := NIL;
  FVersionInfoSize := 0;
  FFilename := '';
  FTranslationIDIndex := 0;
  FForceEXE := FALSE;
  FTranslationIDs := TStringList.Create;
  FFileVersion := TVersionNumberInformation.Create(0, 0);
  FProductVersion := TVersionNumberInformation.Create(0, 0);
  FFixedInfo := TFixedFileVersionInfo.Create(Self);
end;


destructor TVersionInfoResource.Destroy;
begin
  FFileVersion.Free;
  FProductVersion.Free;
  FFixedInfo.Free;
  FTranslationIDs.Free;
  if FVersionInfo <> NIL then
    FreeMem(FVersionInfo, FVersionInfoSize);

  inherited Destroy;
end;


procedure TVersionInfoResource.Loaded;
begin
  inherited Loaded;

  ReadVersionInfoData;
end;


procedure TVersionInfoResource.SetFilename(const Val: string);
begin
  FFilename := Val;
  ReadVersionInfoData;
end;

procedure TVersionInfoResource.ReadVersionInfoData;
const
  TRANSLATION_INFO = '\VarFileInfo\Translation';
type
  TTranslationPair = packed record
    Lang,
    CharSet: word;
  end;
  PTranslationIDList = ^TTranslationIDList;
  TTranslationIDList = array[0..MAXINT div SizeOf(TTranslationPair)-1] of TTranslationPair;
var
  QueryLen: UINT;
  IDsLen: UINT;
  Dummy: DWORD;
  IDs: PTranslationIDList;
  IDCount: integer;
  FixedInfoData: PVSFixedFileInfo;
  TempFilename: array[0..255] of char;
begin
  FTranslationIDs.Clear;
  FFixedInfo.Data := NIL;
  if FVersionInfo <> NIL then
    FreeMem(FVersionInfo, FVersionInfoSize);

  StrPCopy(TempFileName, GetResourceFilename);
  FVersionInfoSize := GetFileVersionInfoSize(TempFileName, Dummy);
  if FVersionInfoSize = 0 then
    FVersionInfo := NIL
  else begin
    GetMem(FVersionInfo, FVersionInfoSize);
    GetFileVersionInfo(TempFileName, Dummy, FVersionInfoSize, FVersionInfo);

    VerQueryValue(FVersionInfo, '\', pointer(FixedInfoData), QueryLen);
    FFixedInfo.Data := FixedInfoData;
    if VerQueryValue(FVersionInfo, TRANSLATION_INFO, Pointer(IDs), IDsLen) then
    begin
      IDCount := IDsLen div SizeOf(TTranslationPair);
      for Dummy := 0 to IDCount-1 do
        FTranslationIDs.Add(Format('%.4x%.4x', [IDs^[Dummy].Lang, IDs^[Dummy].CharSet]));
    end;

    FFileVersion.FMostSignificant := FFixedInfo.GetFileVersionMS;
    FFileVersion.FLeastSignificant := FFixedInfo.GetFileVersionLS;
    FProductVersion.FMostSignificant := FFixedInfo.GetProductVersionMS;
    FProductVersion.FLeastSignificant := FFixedInfo.GetProductVersionLS;
  end;

  if FTranslationIDIndex >= FTranslationIDs.Count then
    FTranslationIDIndex := 0;

  PopulateControls;
end;


procedure TVersionInfoResource.PopulateControls;
begin
  if assigned(FFileVersionLabel) then
    FFileVersionLabel.Caption := FileVersion.AsString;
  if assigned(FCopyrightLabel) then
    FCopyrightLabel.Caption := LegalCopyright;
  if assigned(FProductLabel) then
    FProductLabel.Caption := ProductName;
  if assigned(FDescriptionLabel) then
    FDescriptionLabel.Caption := FileDescription;
  if assigned(FVersionGrid) then
    BuildGrid;
end;


function TVersionInfoResource.GetResourceFilename: string;
var
  TempFilename: array[0..255] of char;
begin
  if FFilename = '' then
  begin
    if IsLibrary and (not FForceEXE) then
    begin
      GetModuleFileName(HInstance, TempFileName, SizeOf(TempFileName)-1);
      Result := StrPas(TempFileName);
    end else
      Result := Application.EXEName;
  end else
    Result := FFilename;
end;


function TVersionInfoResource.GetVersionInfoString(Index: integer): string;
begin
  if (Index >= Low(PREDEF_RESOURCES)) and (Index <= High(PREDEF_RESOURCES)) then
    Result := GetResourceStr(PREDEF_RESOURCES[Index])
  else
    Result := ''
end;


function TVersionInfoResource.GetResourceStr(Index: string): string;
var
  ResStr: PChar;
  StrLen: UINT;
  SubBlock: array[0..255] of char;
  LangCharSet: string;
begin
  if FTranslationIDIndex < FTranslationIDs.Count then
    LangCharSet := FTranslationIDs[FTranslationIDIndex]
  else
    LangCharSet := '';
  StrPCopy(SubBlock, '\StringFileInfo\' + LangCharSet + '\' + Index);
  if (FVersionInfo <> NIL) and
     VerQueryValue(FVersionInfo, SubBlock, Pointer(ResStr), StrLen)
  then
    Result := StrPas(ResStr)
  else
    Result := '';
end;


procedure TVersionInfoResource.SetTranslationIDIndex(Val: integer);
begin
  if (Val > 0) and (Val < FTranslationIDs.Count) then
    FTranslationIDIndex := Val;
end;


function TVersionInfoResource.GetTranslationIDs: TStrings;
begin
  Result := FTranslationIDs;
end;


procedure TVersionInfoResource.SetForceEXE(Val: boolean);
begin
  if FForceEXE <> Val then
  begin
    FForceEXE := Val;
    ReadVersionInfoData;
  end;
end;


procedure TVersionInfoResource.SetFileVersionLabel(Value: TLabel);
begin
  FFileVersionLabel := Value;
  if assigned(FFileVersionLabel) then
  begin
{$IFDEF WIN32}
    FFileVersionLabel.FreeNotification(Self);
{$ENDIF}
    PopulateControls;
  end;
end;

procedure TVersionInfoResource.SetCopyrightLabel(Value: TLabel);
begin
  FCopyrightLabel := Value;
  if assigned(FCopyrightLabel) then
  begin
{$IFDEF WIN32}
    FCopyrightLabel.FreeNotification(Self);
{$ENDIF}
    PopulateControls;
  end;
end;

procedure TVersionInfoResource.SetProductLabel(Value: TLabel);
begin
  FProductLabel := Value;
  if assigned(FProductLabel) then
  begin
{$IFDEF WIN32}
    FProductLabel.FreeNotification(Self);
{$ENDIF}
    PopulateControls;
  end;
end;

procedure TVersionInfoResource.SetDescriptionLabel(Value: TLabel);
begin
  FDescriptionLabel := Value;
  if assigned(FDescriptionLabel) then
  begin
{$IFDEF WIN32}
    FDescriptionLabel.FreeNotification(Self);
{$ENDIF}
    PopulateControls;
  end;
end;

procedure TVersionInfoResource.SetVersionGrid(Value: TStringGrid);
begin
  FVersionGrid := Value;
  if assigned(FVersionGrid) then
  begin
{$IFDEF WIN32}
    FVersionGrid.FreeNotification(Self);
{$ENDIF}
    PopulateControls;
  end;
end;


procedure TVersionInfoResource.Notification(AComponent: TComponent;
   Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if (AComponent = FFileVersionLabel) then
      FFileVersionLabel := NIL
    else if (AComponent = FCopyrightLabel) then
      FCopyRightLabel := NIL
    else if (AComponent = FProductLabel) then
      FProductLabel := NIL
    else if (AComponent = FDescriptionLabel) then
      FDescriptionLabel := NIL
    else if (AComponent = FVersionGrid) then
      FVersionGrid := NIL;
  end;
end;


procedure TVersionInfoResource.BuildGrid;
const
  FLAG_STRING: array[TFixedFileInfoFlag] of string = (
      'Debug', 'Info Inferred', 'Patched', 'Pre-Release', 'Private', 'Special'
    );

  procedure AddGridRow(var RowNum: integer; StrID: integer; Str: string);
  var
    i: integer;
  begin
    if Str <> '' then
    begin
      with VersionGrid do
      begin
        Cells[0,RowNum] := PREDEF_RESOURCES[StrID];
        Cells[1,RowNum] := Str;
        i := Canvas.TextWidth(Str);
        if i > ColWidths[1] then
          ColWidths[1] := i + 4;
        inc(RowNum);
      end;
    end;
  end;

var
  i, FRow: Integer;
  BuildFlags: string;
  AFlag: TFixedFileInfoFlag;
begin
  With VersionGrid do
  begin
    { Set Defaults }
    FixedCols := 0;
    FixedRows := 0;
    ColCount := 2;
    RowCount := 10;
    Canvas.Font.Assign(Font);
    DefaultRowHeight := Canvas.TextHeight(PREDEF_RESOURCES[IDX_ORIGINALFILENAME]) + 2;
    ColWidths[0] := Canvas.TextWidth(PREDEF_RESOURCES[IDX_LEGALTRADEMARKS]) + 4;
    ColWidths[1] := ClientWidth - COlWidths[0] - 1;
    { Clear }
    for i:= 0 to (ColCount-1) do
      Cols[i].Clear;

    FRow := 0;
    if FProductLabel = NIL then
      AddGridRow(FRow, IDX_PRODUCTNAME, ProductName);
    AddGridRow(FRow, IDX_PRODUCTVERSION, ProductVersion.AsString);
    AddGridRow(FRow, IDX_COMPANYNAME, CompanyName);
    if FCopyrightLabel = NIL then
      AddGridRow(FRow, IDX_LEGALCOPYRIGHT, LegalCopyright);
    AddGridRow(FRow, IDX_LEGALTRADEMARKS, LegalTrademarks);
    if FDescriptionLabel = NIL then
      AddGridRow(FRow, IDX_FILEDESCRIPTION, FileDescription);
    if FFileVersionLabel = NIL then
      AddGridRow(FRow, IDX_FILEVERSION, FileVersion.AsString);
    AddGridRow(FRow, IDX_INTERNALNAME, InternalName);
    AddGridRow(FRow, IDX_ORIGINALFILENAME, OriginalFilename);
    BuildFlags := '';
    for AFlag := Low(TFixedFileInfoFlag) to High(TFixedFileInfoFlag) do
    begin
      if AFlag in FixedInfo.Flags then
        BuildFlags := BuildFlags + FLAG_STRING[AFlag] + ', ';
    end;
    if Length(BuildFlags) > 0 then
      BuildFlags := Copy(BuildFlags, 1, Length(BuildFlags)-2);
    AddGridRow(FRow, IDX_BUILDFLAGS, BuildFlags);
(*
    if CompanyName <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_COMPANYNAME];
      Cells[1,FRow] := CompanyName;
      i := Canvas.TextWidth(CompanyName);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
    if InternalName <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_INTERNALNAME];
      Cells[1,FRow] := InternalName;
      i := Canvas.TextWidth(InternalName);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
    if ProductName <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_PRODUCTNAME];
      Cells[1,FRow] := ProductName;
      i := Canvas.TextWidth(ProductName);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
    if ProductVersion.AsString <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_PRODUCTVERSION];
      Cells[1,FRow] := ProductVersion.AsString;
      i := Canvas.TextWidth(ProductVersion.AsString);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
    if OriginalFileName <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_ORIGINALFILENAME];
      Cells[1,FRow] := OriginalFileName;
      i := Canvas.TextWidth(OriginalFileName);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
    if LegalTradeMarks <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_LEGALTRADEMARKS];
      Cells[1,FRow] := LegalTradeMarks;
      i := Canvas.TextWidth(LegalTradeMarks);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;

    BuildFlags := '';
    for AFlag := Low(TFixedFileInfoFlag) to High(TFixedFileInfoFlag) do
    begin
      if AFlag in FixedInfo.Flags then
        BuildFlags := BuildFlags + FLAG_STRING[AFlag] + ', ';
    end;
    if Length(BuildFlags) > 0 then
    begin
      BuildFlags := Copy(BuildFlags, 1, Length(BuildFlags)-2);
      Cells[0,FRow] := 'BuildFlags';
      Cells[1,FRow] := BuildFlags;
      i := Canvas.TextWidth(BuildFlags);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;

    if Comments <> '' then
    begin
      Cells[0,FRow] := PREDEF_RESOURCES[IDX_COMMENTS];
      Cells[1,FRow] := Comments;
      i := Canvas.TextWidth(Comments);
      if i > ColWidths[1] then
        ColWidths[1] := i + 4;
      Inc(FRow);
    end;
*)
    RowCount := FRow;
  end;
end;


end.
